//
//  GameScene.swift
//  ex_SKAction_IK
//
//  Created by yoshiyuki oshige on 2014/10/23.
//  Copyright (c) 2014年 yoshiyuki oshige. All rights reserved.
//

import SpriteKit

class GameScene: SKScene {
    // SKViewに追加されたら実行される
    override func didMoveToView(view: SKView) {
        // スプライトノードで腕を作る
        let arm1 = SKSpriteNode(color: SKColor.whiteColor(),
            size: CGSizeMake(15,80))
        let arm2 = SKSpriteNode(color: SKColor.whiteColor(),
            size: CGSizeMake(10,150))
        // ノードに名前をつける
        arm1.name = "arm1"
        arm2.name = "arm2"
        // アンカーポイント（基準点）を腕の端に設定する
        arm1.anchorPoint = CGPointMake(0.5, 0.95)
        arm2.anchorPoint = CGPointMake(0.5, 0.95)
        // 回転の許容範囲（ -60度〜60度）
        let angleLimit = SKReachConstraints(
            lowerAngleLimit: CGFloat(-M_PI*2/3),
            upperAngleLimit: CGFloat(M_PI*2/3))
        arm1.reachConstraints = angleLimit
        arm2.reachConstraints = angleLimit
        
        // シェイプノードで手を作る
        let hand = SKShapeNode(ellipseOfSize: CGSizeMake(30, 20))
        hand.fillColor = SKColor.whiteColor()
        // ノードに名前をつける
        hand.name = "hand"
        
        // 表示座標
        let x = view.frame.width/2
        arm1.position = convertPointFromView(CGPointMake(x, 200))
        arm2.position = CGPointMake(0,-90)
        hand.position = CGPointMake(0,-160)
        // arm1をシーンに追加する
        self.addChild(arm1)
        // arm2をarm1の子ノードにする
        arm1.addChild(arm2)
        // handをarm2の子ノードにする
        arm2.addChild(hand)
    }
    
    // タッチ開始
    override func touchesBegan(touches: Set<NSObject>, withEvent event: UIEvent) {
        armAction(touches, withEvent: event)
    }
    // タッチしたまま動かす
    override func touchesMoved(touches: Set<NSObject>, withEvent event: UIEvent) {
        armAction(touches, withEvent: event)
    }
    
    // タッチ座標にhand-arm2-arm1を回転する
    func armAction(touches: NSSet, withEvent event: UIEvent) {
        // ノードを取得する
        let arm1 = childNodeWithName("arm1")
        let arm2 = arm1?.childNodeWithName("arm2") as? SKSpriteNode
        let hand = arm2?.childNodeWithName("hand") as? SKShapeNode
        
        // タップされた位置にhandを回す
        for touch: AnyObject in touches {
            // タップされた座標locを取り出す
            let loc = touch.locationInNode(self)
            // 座標locの角度に向けてhandを回転させる（インバースキネマティック）
            let action = SKAction.reachTo(loc, rootNode: arm1!, duration: 0.4)
            hand?.runAction(action)
        }
    }
}