//
//  GameScene.swift
//  ex_SKPhysicsJointSpring
//
//  Created by yoshiyuki oshige on 2014/10/24.
//  Copyright (c) 2014年 yoshiyuki oshige. All rights reserved.
//

import SpriteKit

class GameScene: SKScene {
    
    override func didMoveToView(view: SKView) {
        // Tシャツのスプライトノードを作る
        let tshirt = SKSpriteNode(imageNamed: "T-shirt")
        tshirt.setScale(0.5)
        // 物理ボディを設定する
        tshirt.physicsBody = SKPhysicsBody(circleOfRadius: tshirt.size.height/2)
        tshirt.physicsBody?.friction = 0.9
        tshirt.physicsBody?.restitution = 0.6
        tshirt.position = convertPointFromView(CGPointMake(60,-10))
        self.addChild(tshirt)
 
        // カエルのスプライトノードを作る
        let frog = SKSpriteNode(imageNamed: "frog")
        frog.setScale(0.2)
        // 物理ボディを設定する
        frog.physicsBody = SKPhysicsBody(circleOfRadius: frog.size.height/2)
        frog.position = tshirt.position  // Tシャツと重なる位置
        self.addChild(frog)
        
        // スプリングの物理ジョイントを作る
        let spring = SKPhysicsJointSpring.jointWithBodyA(
            tshirt.physicsBody,
            bodyB: frog.physicsBody,
            anchorA: tshirt.position,
            anchorB: frog.position)
        spring.damping = 0.1
        spring.frequency = 6
        
        // ジョイントの長さ
        let limit = SKPhysicsJointLimit.jointWithBodyA(
            tshirt.physicsBody,
            bodyB: frog.physicsBody,
            anchorA: tshirt.position,
            anchorB: frog.position)
        limit.maxLength = 50
        
        // ジョイントの滑り
        let sliding = SKPhysicsJointSliding.jointWithBodyA(
            tshirt.physicsBody,
            bodyB: frog.physicsBody,
            anchor: tshirt.position,
            axis: CGVectorMake(0, 0))

        // スプリングを追加
        self.physicsWorld.addJoint(spring)
        // リミットを追加
        self.physicsWorld.addJoint(limit)
        // スライディングを追加（Tシャツの回転に合わせてカエルも回転する）
        self.physicsWorld.addJoint(sliding)
    }

}
