//
//  GameScene.swift
//  ex_categoryBitMask_collisionBitMask
//
//  Created by yoshiyuki oshige on 2014/10/27.
//  Copyright (c) 2014年 yoshiyuki oshige. All rights reserved.
//

import SpriteKit

// カテゴリを定義する
enum ColCategory: UInt32 {
    case ellipse = 0b1
    case circle = 0b10
    case box = 0b100
}

class GameScene: SKScene {
    override func didMoveToView(view: SKView) {
        // 重力加速度
        self.physicsWorld.gravity = CGVectorMake(0, -4)
        // 物理ボディの有効領域
        let worldRect = CGRectMake(300, 0, 430, 768)
        self.physicsBody = SKPhysicsBody(edgeLoopFromRect: worldRect)
        
        // 楕円形のノード
        let ellipse = SKShapeNode(ellipseOfSize: CGSizeMake(180, 60))
        ellipse.lineWidth = 4
        ellipse.zRotation = CGFloat(M_PI/180*10)
        ellipse.physicsBody = SKPhysicsBody(polygonFromPath: ellipse.path)
        ellipse.physicsBody?.restitution = 0.6
        // 円形のノード
        let circle1 = SKShapeNode(circleOfRadius: 40)
        circle1.lineWidth = 4
        circle1.physicsBody = SKPhysicsBody(circleOfRadius: circle1.frame.width/2)
        circle1.physicsBody?.restitution = 0.6
        let circle2 = SKShapeNode(circleOfRadius: 80)
        circle2.lineWidth = 4
        circle2.physicsBody = SKPhysicsBody(circleOfRadius: circle2.frame.width/2)
        circle2.physicsBody?.restitution = 0.6
        // 四角形のノード
        let box1 = SKShapeNode(rectOfSize: CGSizeMake(40, 40))
        box1.lineWidth = 4
        box1.zRotation = CGFloat(M_PI/180*10)
        box1.physicsBody = SKPhysicsBody(rectangleOfSize: box1.frame.size)
        box1.physicsBody?.restitution = 0.6
        let box2 = SKShapeNode(rectOfSize: CGSizeMake(80, 100))
        box2.lineWidth = 4
        box2.zRotation = CGFloat(M_PI/180*15)
        box2.physicsBody = SKPhysicsBody(rectangleOfSize: box2.frame.size)
        box2.physicsBody?.restitution = 0.6
        let box3 = SKShapeNode(rectOfSize: CGSizeMake(70, 150))
        box3.lineWidth = 4
        box3.zRotation = CGFloat(-M_PI/180*10)
        box3.physicsBody = SKPhysicsBody(rectangleOfSize: box3.frame.size)
        box3.physicsBody?.restitution = 0.6
        
        // 物理ボディが属するカテゴリ
        ellipse.physicsBody?.categoryBitMask = ColCategory.ellipse.rawValue
        circle1.physicsBody?.categoryBitMask = ColCategory.circle.rawValue
        circle2.physicsBody?.categoryBitMask = ColCategory.circle.rawValue
        box1.physicsBody?.categoryBitMask = ColCategory.box.rawValue
        box2.physicsBody?.categoryBitMask = ColCategory.box.rawValue
        box3.physicsBody?.categoryBitMask = ColCategory.box.rawValue
        
        // 衝突する相手のカテゴリを決める
        ellipse.physicsBody?.collisionBitMask = ColCategory.box.rawValue | ColCategory.circle.rawValue
        circle1.physicsBody?.collisionBitMask = ColCategory.box.rawValue | ColCategory.ellipse.rawValue
        circle2.physicsBody?.collisionBitMask = ColCategory.box.rawValue | ColCategory.ellipse.rawValue
        box1.physicsBody?.collisionBitMask = ColCategory.circle.rawValue | ColCategory.ellipse.rawValue
        box2.physicsBody?.collisionBitMask = ColCategory.circle.rawValue | ColCategory.ellipse.rawValue
        box3.physicsBody?.collisionBitMask = ColCategory.circle.rawValue | ColCategory.ellipse.rawValue

        // シーンにノードを追加する
        let nodes = [ellipse, circle1, circle2, box1, box2, box3]
        for node in nodes {
            let px = CGFloat(arc4random_uniform(300)+50)
            let py = CGFloat(arc4random_uniform(300))
            node.position = convertPointFromView(CGPointMake(px, py))
            self.addChild(node)
        }
    }
    
}
