//
//  GameScene.swift
//  ex_categoryBitMask_collisionBitMask
//
//  Created by yoshiyuki oshige on 2014/10/27.
//  Copyright (c) 2014年 yoshiyuki oshige. All rights reserved.
//

import SpriteKit

class GameScene: SKScene {
    
    // カテゴリを定義する
    enum Category: UInt32 {
        case ellipse = 0b1
        case circle = 0b10
        case box = 0b100
    }
    
    override func didMoveToView(view: SKView) {
        // 重力加速度
        self.physicsWorld.gravity = CGVectorMake(0, -4)
        // 物理ボディの有効領域
        let worldRect = CGRectMake(300, 0, 430, 768)
        self.physicsBody = SKPhysicsBody(edgeLoopFromRect: worldRect)
        
        /* 楕円形のノード */
        let ellipse1 = SKShapeNode(ellipseOfSize: CGSizeMake(150, 50))
        ellipse1.lineWidth = 2
        ellipse1.zRotation = CGFloat(M_PI/180*10) //傾ける
        // 楕円形の物理ボディを作る
        ellipse1.physicsBody = SKPhysicsBody(polygonFromPath: ellipse1.path)
        ellipse1.physicsBody?.restitution = 0.6
        let ellipse2 = SKShapeNode(ellipseOfSize: CGSizeMake(250, 90))
        ellipse2.lineWidth = 2
        ellipse2.zRotation = CGFloat(-M_PI/180*10)
        ellipse2.physicsBody = SKPhysicsBody(polygonFromPath: ellipse2.path)
        ellipse2.physicsBody?.restitution = 0.6
        
        /* 円形のノード */
        let circle1 = SKShapeNode(circleOfRadius: 40)
        circle1.lineWidth = 2
        // 円形の物理ボディを作る
        circle1.physicsBody = SKPhysicsBody(circleOfRadius: circle1.frame.width/2)
        circle1.physicsBody?.restitution = 0.6
        let circle2 = SKShapeNode(circleOfRadius: 80)
        circle2.lineWidth = 2
        circle2.physicsBody = SKPhysicsBody(circleOfRadius: circle2.frame.width/2)
        circle2.physicsBody?.restitution = 0.6
        
        /* 四角形のノード */
        let box1 = SKShapeNode(rectOfSize: CGSizeMake(40, 140))
        box1.lineWidth = 3
        // 四角形の物理ボディを作る
        box1.physicsBody = SKPhysicsBody(rectangleOfSize: box1.frame.size)
        box1.physicsBody?.restitution = 0.6
        let box2 = SKShapeNode(rectOfSize: CGSizeMake(80, 200))
        box2.lineWidth = 3
        box2.physicsBody = SKPhysicsBody(rectangleOfSize: box2.frame.size)
        box2.physicsBody?.restitution = 0.6
        
        // 物理ボディが属するカテゴリ
        ellipse1.physicsBody?.categoryBitMask = Category.ellipse.rawValue
        ellipse2.physicsBody?.categoryBitMask = Category.ellipse.rawValue
        circle1.physicsBody?.categoryBitMask = Category.circle.rawValue
        circle2.physicsBody?.categoryBitMask = Category.circle.rawValue
        box1.physicsBody?.categoryBitMask = Category.box.rawValue
        box2.physicsBody?.categoryBitMask = Category.box.rawValue
        
        // 衝突する相手のカテゴリを指定する
        ellipse1.physicsBody?.collisionBitMask = Category.box.rawValue
        ellipse2.physicsBody?.collisionBitMask = Category.box.rawValue
        circle1.physicsBody?.collisionBitMask = Category.box.rawValue
        circle2.physicsBody?.collisionBitMask = Category.box.rawValue
        box1.physicsBody?.collisionBitMask =
            (Category.box.rawValue | Category.ellipse.rawValue | Category.circle.rawValue)
        box2.physicsBody?.collisionBitMask =
            (Category.box.rawValue | Category.ellipse.rawValue | Category.circle.rawValue)
        
        // シーンにノードを追加する
        let nodes = [ellipse1, ellipse2, circle1, circle2, box1, box2]
        for node in nodes {
            let px = CGFloat(arc4random_uniform(300)+50)
            let py = CGFloat(arc4random_uniform(300))
            node.position = convertPointFromView(CGPointMake(px, py))
            self.addChild(node)
        }
    }
    
}
