//
//  GameScene.swift
//  ex_contactTestBitMask
//
//  Created by yoshiyuki oshige on 2014/10/27.
//  Copyright (c) 2014年 yoshiyuki oshige. All rights reserved.
//

import SpriteKit
import UIKit

class GameScene: SKScene, SKPhysicsContactDelegate {
    // カテゴリを定義する
    enum Category: UInt32 {
        case ellipse = 0b1
        case circle = 0b10
        case paddle = 0b100
    }
    // シェイプノード
    var ellipse = SKShapeNode()
    var circle = SKShapeNode()
    var paddle = SKShapeNode()
    
    override func didMoveToView(view: SKView) {
        // SKPhysicsContactDelegateクラスのデリゲートになる
        self.physicsWorld.contactDelegate = self
        
        // マルチタップを受け付けない
        view.multipleTouchEnabled = false
        // 重力加速度
        self.physicsWorld.gravity = CGVectorMake(0, -2)
        // 物理ボディの有効領域
        let worldRect = CGRectMake(300, 0, 430, 768)
        self.physicsBody = SKPhysicsBody(edgeLoopFromRect: worldRect)
        
        // 楕円形のノード
        ellipse = SKShapeNode(ellipseOfSize: CGSizeMake(60, 40))
        ellipse.name = "ellipse"
        ellipse.lineWidth = 8
        ellipse.strokeColor = SKColor.redColor()
        ellipse.zRotation = CGFloat(M_PI/180*10)
        ellipse.physicsBody = SKPhysicsBody(polygonFromPath: ellipse.path)
        ellipse.physicsBody?.restitution = 0.5
        ellipse.physicsBody?.friction = 0.6
        // 円形のノード
        circle = SKShapeNode(circleOfRadius: 20)
        circle.name = "circle"
        circle.lineWidth = 8
        circle.strokeColor = SKColor.yellowColor()
        circle.physicsBody = SKPhysicsBody(circleOfRadius: circle.frame.width/2)
        circle.physicsBody?.restitution = 0.9
        circle.physicsBody?.friction = 0.6
        // パドルのノード
        paddle = SKShapeNode(rectOfSize: CGSizeMake(100, 10))
        paddle.name = "paddle"
        paddle.fillColor = SKColor.whiteColor()
        paddle.strokeColor = SKColor.clearColor()
        // 物理ボディは楕円にする
        let ovalPath = UIBezierPath(ovalInRect: paddle.frame).CGPath
        paddle.physicsBody = SKPhysicsBody(polygonFromPath: ovalPath)
        paddle.physicsBody?.restitution = 1.0
        paddle.physicsBody?.friction = 0.6
        paddle.physicsBody?.dynamic = false
        
        // 物理ボディが属するカテゴリ
        ellipse.physicsBody?.categoryBitMask = Category.ellipse.rawValue
        circle.physicsBody?.categoryBitMask = Category.circle.rawValue
        paddle.physicsBody?.categoryBitMask = Category.paddle.rawValue
        
        // 衝突する相手のカテゴリを決める
        ellipse.physicsBody?.collisionBitMask =
            Category.paddle.rawValue | Category.circle.rawValue
        circle.physicsBody?.collisionBitMask =
            Category.paddle.rawValue | Category.ellipse.rawValue
        paddle.physicsBody?.collisionBitMask =
            Category.circle.rawValue | Category.ellipse.rawValue
        
        // 衝突したならば通知する衝突相手を設定する
        paddle.physicsBody?.contactTestBitMask =
            Category.circle.rawValue | Category.ellipse.rawValue
        
        // シーンにノードを追加する
        paddle.position = CGPointMake(CGRectGetMidX(self.frame), 180)
        addChild(paddle)
        let nodes = [ellipse, circle]
        for node in nodes {
            let px = CGFloat(arc4random_uniform(300)+50)
            let py = CGFloat(arc4random_uniform(300))
            node.position = convertPointFromView(CGPointMake(px, py))
            self.addChild(node)
        }
    }
    
    // 衝突によるデリゲートメソッド
    func didBeginContact(contact: SKPhysicsContact) {
        // 衝突した物理ボディに衝撃を加える
        contact.bodyB.applyAngularImpulse(0.1)
        // パドルをbodyB（当たってきた側）の色に変える
        switch contact.bodyB.categoryBitMask {
            case Category.ellipse.rawValue:
                paddle.fillColor = ellipse.strokeColor
            case Category.circle.rawValue:
                paddle.fillColor = circle.strokeColor
        default:
            println("default ?")
        }
    }
    
    // パドルをドラッグで動かす
    override func touchesMoved(touches: Set<NSObject>, withEvent event: UIEvent) {
        for touch: AnyObject in touches {
            let location = touch.locationInNode(self)
            // 水平方向だけタッチの位置に動かす
            paddle.position = CGPointMake(location.x, CGFloat(paddle.position.y))
        }
    }
    
}

